﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BastaAdministrator.ViewModel
{
    using System.Collections.ObjectModel;
    using System.ComponentModel;
    using System.Runtime.CompilerServices;
    using System.Threading;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Input;

    using BastaAdministrator.Annotations;
    using BastaAdministrator.DataProvider;
    using System.Diagnostics;

    public class MainViewModel : ViewModelBase
    {
        private readonly ISpeakerProvider _speakerProvider;

        private readonly ISessionProvider _sessionProvider;

        private string _ellapsedMilliseconds;

        private bool _isLoading;

        private int _loadingProgress;

        public MainViewModel(ISpeakerProvider speakerProvider,
            ISessionProvider sessionProvider)
        {
            this._speakerProvider = speakerProvider;
            this._sessionProvider = sessionProvider;

            Speakers = new ObservableCollection<string>();
            Sessions = new ObservableCollection<string>();

            LoadCommand = new DelegateCommand(OnLoadExecute, OnLoadCanExecute);
            CancelCommand = new DelegateCommand(OnCancelExecute);
        }

        public DelegateCommand CancelCommand { get; set; }

        private Task OnCancelExecute(object arg)
        {
            if (_cts != null && !_cts.IsCancellationRequested)
            {
                _cts.Cancel();
                _cts = null;
            }
            return Task.Delay(0);
        }

        private bool OnLoadCanExecute(object arg)
        {
            return !IsLoading;
        }

        private CancellationTokenSource _cts;
        private async Task OnLoadExecute(object obj)
        {
            _cts = new CancellationTokenSource();
            IsLoading = true;
            var watch = Stopwatch.StartNew();
            var speakersTask = _speakerProvider.LoadSpeakersAsync(Report, _cts.Token);
            var sessionsTask = _sessionProvider.LoadSessionsAsync();

            var speakers = await speakersTask;
            var sessions = await sessionsTask;

            watch.Stop();

            EllapsedMilliseconds = string.Format("{0}ms", watch.ElapsedMilliseconds);

            Speakers.Clear();
            foreach (var speaker in speakers)
            {
                Speakers.Add(speaker);
            }

            Sessions.Clear();
            foreach (var session in sessions)
            {
                Sessions.Add(session);
            }

            IsLoading = false;
        }

        public bool IsLoading
        {
            get
            {
                return this._isLoading;
            }
            set
            {
                this._isLoading = value;
                this.OnPropertyChanged();
                ((DelegateCommand)LoadCommand).RaiseCanExecuteChanged();
            }
        }

        public string EllapsedMilliseconds
        {
            get
            {
                return this._ellapsedMilliseconds;
            }
            set
            {
                this._ellapsedMilliseconds = value;
                this.OnPropertyChanged();
            }
        }

        public IAsyncCommand LoadCommand { get; set; }

        public ObservableCollection<string> Speakers { get; set; }
        public ObservableCollection<string> Sessions { get; set; }

        public void Report(int value)
        {
        
            LoadingProgress = value;
        }

        public int LoadingProgress
        {
            get
            {
                return this._loadingProgress;
            }
            set
            {
                this._loadingProgress = value;
                this.OnPropertyChanged();
            }
        }
    }

    public class ViewModelBase : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(
            [CallerMemberName] string propertyName = null)
        {
            var handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }
}
